#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
//#include "hofmeier.h"
#include <stdio.h>
#include <asm/io.h>
#include <stdlib.h>
#include <time.h>

int PORT;
#define SLP 0
#define SLPC 1
#define SLPD 0

/*
  0 = /dev/lp0          First parallel printer (0x3bc)
  1 = /dev/lp1          Second parallel printer (0x378)
  2 = /dev/lp2          Third parallel printer (0x278)  */

/* Binrcode */
unsigned char binar[] = {1, 2, 4, 8, 16, 32, 64, 128};

/* 
   z = 1 : Schreibe Zeichen in Ram
       0 : Commando fr LCD 
   l = 4 : Bitlnge 4 / Halbes Byte (2 Schritte)
       8 : Bitlnge 8 / ein Byte (1 Schritt)
   s = Data */
lcd_write(z, l, s)
  unsigned char z, l, s;
{
  unsigned char x;

  /* 4 Datenbitz ... */
  if (l == 4) {
    x = 0;
    /* ** 1 ** RS Bit bei bedarf setzen */
    if (z == 1)	{
      x += 16;
    }
    outb(x, PORT);
    usleep(SLP);
    /* ** 2 ** E - Steigende Flanke */
    x += 32;
    outb(x, PORT);
    usleep(SLP);
    /* ** 3 ** Daten werden angelegt */
    x += (s & 15);
    outb(x, PORT);
    usleep(SLP);
    /* ** 4 ** E - Fallende Flanke (Daten werden bernommen)*/
    x -= 32;
    outb(x, PORT);
    usleep(SLP);
    /* ** 5 ** Reset */
    x = 0;
    outb(x, PORT);
    usleep(SLP);
    usleep(SLPC);
    /* EXIT */
    return 0;
  }

  /* 8 Datenbitz ... zwei Durchgnge */
  if (l == 8) {
    x = 0;
    /* ########## ERSTER SCHRITT ########## */
    /* ** 1 ** RS Bit bei bedarf setzen */
    if (z == 1) {
      x += 16;
    }
    outb(x, PORT);
    usleep(SLP);
    /* ** 2 ** E - Steigende Flanke */
    x += 32;
    outb(x, PORT);
    usleep(SLP);
    /* ** 3 ** Daten werden angelegt */
    x += ((s >> 4) & 15);
    outb(x, PORT);
    usleep(SLP);
    /* ** 4 ** E - Fallende Flanke (Daten werden bernommen)*/
    x -= 32;
    outb(x, PORT);
    usleep(SLP);
    /* ** 5 ** Reset */
    x = 0;
    outb(x, PORT);
    usleep(SLP);
    //      usleep(SLPC);
    /* ########## Zweiter SCHRITT ########## */
    /* ** 1 ** RS Bit bei bedarf setzen */
    if (z == 1)	{
      x += 16;
    }
    outb(x, PORT);
    usleep(SLP);
    /* ** 2 ** E - Steigende Flanke */
    x += 32;
    outb(x, PORT);
    usleep(SLP);
    /* ** 3 ** Daten werden angelegt */
    x += (s & 15);
    outb(x, PORT);
    usleep(SLP);
    /* ** 4 ** E - Fallende Flanke (Daten werden bernommen)*/
    x -= 32;
    outb(x, PORT);
    usleep(SLP);
    /* ** 5 ** Reset */
    x = 0;
    outb(x, PORT);
    usleep(SLP);
    usleep(SLPC);
    /* EXIT */
    return 0;
  }
}


/* Giebt eine Zeichenkette auf dem LCD aus... */
lcd_write_str(s)
  char *s;
{
  int i, ix;

  ix = strlen(s);
  for (i = 0; i < ix; i++) {
    switch(s[i]) {
    case '':
    case '':
      lcd_write(1, 8, 225);
      break;
    case '':
    case '':
      lcd_write(1, 8, 245);
      break;
    case '':
    case '':
      lcd_write(1, 8, 239);
      break;
    case '':
      lcd_write(1, 8, 226);
      break;
    default:
      lcd_write(1, 8, s[i]);
      break;
    }
  }
  lcd_write(0, 8, 2); /* <<-- */
}


/* Manche 16-Zeichen LCD (1Zeile) sind intern auf zwei 8Zeichen Zeilen
   aufgeteilt. Diese Procedur beschreibt es Komplett. 
*/
x_lcd_16(s)
  char *s;
{
  int i, ix;

  ix = strlen(s);
  for (i = 0; i < ix; i++) {
    if (i == 8) {
      /* Zweite Zeile / Zweiten 8 Zeichen */
      lcd_write(0, 8, 192);	  
    }
    switch(s[i]) { // Hm, manche Zeichen haben andere Nummern
    case '':
    case '':
      lcd_write(1, 8, 225);
      break;
    case '':
    case '':
      lcd_write(1, 8, 245);
      break;
    case '':
    case '':
      lcd_write(1, 8, 239);
      break;
    case '':
      lcd_write(1, 8, 226);
      break;
    default:
      lcd_write(1, 8, s[i]);
      break;
    } /* switch() */
  }
  lcd_write(0, 8, 2); /* <<-- */
}


lcd_init()
{
  lcd_write(0, 4, 2);   /* 4-Bit Interface */
  usleep(SLPD);
  //lcd_write(0, 8, 32+2);  /* 1-Line/5x8 Dot */
  usleep(SLPD);
  //  lcd_write(0, 8, 14);  /* Turn On Display */
  lcd_write(0, 8, 12); /* ... Ohne Cursor... */
  usleep(SLPD);
  //  lcd_write(0, 8, 6);   /* ... */
  lcd_write(0, 8, 2);
  usleep(SLPD);
}


main(argc, argv)
  int argc;
  char *argv[];
{
  int i, ix;
  char s[300], *ss;
  long ti, tii;
  struct tm *tminfo;

  //Parameter
  if (argc < 2) {
    printf("lcd <PORT inn Hex> [-i] [text to line 0] [text to line 1]\nz.B, lcd 378\n");
    exit(0);
  }

  if (getuid() != 0) {
    fprintf(stderr, "** ERROR: Dieses Programm mu unter root laufen, da es sonnst nicht auf die HardWare zugreifen kann!\n");
    //    exit(0);
  }


  //PORT=atoi(argv[1]);
  sscanf(argv[1], "%x", &PORT);
  printf("PORT: 0x%x\n", PORT);

  // Zugriff auf HardWare-Port (nur unter root)
  if (iopl(3) < 0) {
    perror("iopl(3)");
    exit(1);
  }

  // Command
  if (argc > 2) {
    if ((argv[2][0] == '-') && (argv[2][1] == 'i') && (argv[2][2] == 0)) {
      lcd_init();
      if (argc > 3) {
	lcd_write(0, 8, 2); /* <<-- */
	lcd_write_str(argv[3]);
	if (argc > 4) {
	  lcd_write(0, 8, 192);      
	  lcd_write_str(argv[4]);
	}
      }
    } else {
      lcd_write(0, 8, 2); /* <<-- */
      lcd_write_str(argv[2]);
      if (argc > 3) {
	lcd_write(0, 8, 192);      
	lcd_write_str(argv[3]);
      }
    }
    exit(0);
  }


  // Online-modus

/* z = 1 : Schreibe Zeichen in Ram
       0 : Commando fr LCD 
   l = 4 : Bitlnge 4 / Halbes Byte (2 Schritte)
       8 : Bitlnge 8 / ein Byte (1 Schritt)
   s = Data */
  /*lcd_write(z, l, s)*/
  lcd_init();
    
  /* Zweite Zeile */
  //  lcd_write(0, 8, 192);

  lcd_write(0, 8, 2); /* <<-- */

  strcpy(s, "Andreas Hofmeier");

  for (  ; s[0] != 0;  ) {
    x_lcd_16(s);
    printf("Write to LCD>");
    gets(s);
    lcd_write(0, 8, 1);
  }

  usleep(SLPD);



  for (  ;  ;  ) {
    usleep(5);

    ti = time(&ti);

    if (ti != tii) {
      char *ss;
      /* Liste, welche Stellen durch 0 ersetzt werden sollen, wenn 
	 Sie lehr sind..., mit 255 Terminieren...*/
      int xx[] = {0, 3, 6, 10, 13, 255};
      
      tii = ti;
      
      tminfo = localtime(&ti);
#if 0
      year = (tminfo -> tm_year) + 1900; /* Jahr        */
      mon = (tminfo -> tm_mon) + 1;      /* Monat       */
      day = (tminfo -> tm_mday);         /* Tag         */
      dow = (tminfo -> tm_wday);         /* Wochentag   */
      stu = (tminfo -> tm_hour);         /* Stunde      */
      min = (tminfo -> tm_min);          /* Minute      */
      doy = (tminfo -> tm_yday);         /* Tag im Jahr */
#endif
      
      ss = ctime(&ti);
      /* Zeit, Datum in String */
      sprintf(s, "%02d:%02d:%02d  %02d.%02d.", (tminfo -> tm_hour), (tminfo ->
								tm_min),
	      (tminfo -> tm_sec), (tminfo -> tm_mday), (tminfo ->
							tm_mon) +
	      1);
      /* String auf LCD */
      ss = s;
      x_lcd_16(ss);      /* 16Zeichen LCD */
      //	  lcd_write_ss); /* Fr Normale LCDs */
    }
  }
}

/*

LCD Pins:
  Pin  1 - GND
  Pin  2 - +5V
  Pin  3 - LCD Contrast - 0V to +5V
  Pin  4 - R/S (Data/Instruction Select)
  Pin  5 - R/W (Data Read/Write Select)
  Pin  6 - E   (Data/Instruction Strobe)
  Pin  7 - Data Bit 0
  Pin  8 - Data Bit 1
  Pin  9 - Data Bit 2
  Pin 10 - Data Bit 3
  Pin 11 - Data Bit 4
  Pin 12 - Data Bit 5
  Pin 13 - Data Bit 6
  Pin 14 - Data Bit 7

4-Bit-Verbindung:
        l     g
        a     n
        r  D  u
        a  C  t
        P  L  u
  t           e
  r  t  N  N  d
  e  i  I  I  e
  W  B  P  P  B
---------------
  1  0  2 11  Data Bit 4
  2  1  3 12  Data Bit 5
  4  2  4 13  Data Bit 6
  8  3  5 14  Data Bit 7
 16  4  6  4  RS (1=Schreibe in RAM // 0=Commando)
 32  5  7  6  E  (Clock - Adress Enable)
GND        1  GND
+5V        2  +5V
GND        3  Contrast
GND        5  R/-W (0=Write // 1=Read)

Schritte zum Beschreiben:
         1   2   3   4   5     # 1: RS Bit bei bedarf setzen
---------|---|---|---|---|---  # 2: E - Steigende Flanke
RS   ____|###|###|###|###|___  # 3: Daten werden angelegt
R/-W ____|___|___|___|___|___  # 4: E - Fallende Flanke
E    ____|___|###|###|___|___  #    (Daten werden bernommen)
Data ____|___|___|###|###|___  # 5: Daten/RS - Reset


Commands:   1: Clear Display
            2: Cursor in Ausgangsstellung... 
	   16: Rcksritt
          192: Zweite Zeile

	  00001DCB(2): On/Off
	       D: Display
	       C: Cursor
	       B: Blink-Cursor
	  
 */
